import 'package:firebase_hooks_core/src/models/async_snapshot_with_progress.dart';
import 'package:flutter/material.dart';

extension WhenAsyncSnapshotWithProgressExtension<T, ProgressType extends Object>
    on AsyncSnapshotWithProgress<T, ProgressType> {
  R when<R>({
    required R Function(T data) data,
    required R Function(Object, StackTrace?, T? previous) error,
    required R Function(ProgressType progress, T? previous) loading,
    required R Function() nothing,
  }) {
    if (hasError) {
      return error(this.error!, stackTrace, this.data);
    }

    if (connectionState == ConnectionState.active ||
        connectionState == ConnectionState.waiting) {
      return loading(progress!, this.data);
    }

    if (hasData) {
      return data(this.data!);
    }

    return nothing();
  }

  R maybeWhen<R>({
    R Function(T data)? data,
    R Function(Object, StackTrace?, T? previous)? error,
    R Function(ProgressType progress, T? previous)? loading,
    R Function()? nothing,
    required R Function() orElse,
  }) {
    if (hasError && error != null) {
      return error(this.error!, stackTrace, this.data);
    }

    if ((connectionState == ConnectionState.active ||
            connectionState == ConnectionState.waiting) &&
        loading != null) {
      return loading(progress!, this.data);
    }

    if (hasData && data != null) {
      return data(this.data!);
    }

    if (nothing != null) {
      return nothing();
    }

    return orElse();
  }
}
